import { MySQLColumn } from './mysql-column';
import { Table } from '../table-descriptor.interface';
export interface ForeignKey {
    column: string;
    foreignTable: string;
    foreignColumn: string;
    uniqueIndex: boolean;
}
export interface DatabaseConnector {
    init(): Promise<void>;
    getTablesInformation(ignoredTables: string[], tablesToFill: string[]): Promise<Table[]>;
    getColumnsInformation(table: Table): Promise<MySQLColumn[]>;
    getForeignKeys(table: Table): Promise<ForeignKey[]>;
    countLines(table: Table): Promise<number>;
    emptyTable(table: Table): Promise<void>;
    getValuesForForeignKeys(table: string, column: string, foreignTable: string, foreignColumn: string, limit: number, unique: boolean, condition: string | undefined): Promise<any[]>;
    executeRawQuery(query: string): Promise<void>;
    insert(table: string, lines: any[]): Promise<number>;
    destroy(): Promise<void>;
    backupTriggers(tables: string[]): Promise<void>;
    cleanBackupTriggers(): void;
    disableTriggers(table: string): Promise<void>;
    enableTriggers(table: string): Promise<void>;
}
export declare enum databaseEngine {
    'MariaDB' = 0
}
export declare class DatabaseConnectorBuilder {
    private engine;
    private ip;
    private port;
    private database;
    private user;
    private password;
    constructor(engine: databaseEngine);
    build(): Promise<DatabaseConnector>;
    setHost(ip: string): this;
    setPort(port: number): this;
    setDatabase(database: string): this;
    setCredentials(user: string, password: string): this;
}
