"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DatabaseConnectorBuilder = exports.databaseEngine = void 0;
const mariadb_connector_1 = require("./mariadb-connector");
var databaseEngine;
(function (databaseEngine) {
    databaseEngine[databaseEngine["MariaDB"] = 0] = "MariaDB";
})(databaseEngine = exports.databaseEngine || (exports.databaseEngine = {}));
class DatabaseConnectorBuilder {
    constructor(engine) {
        this.engine = engine;
        this.ip = '127.0.0.1';
        this.port = 3306;
        this.database = '';
        this.user = '';
        this.password = '';
    }
    build() {
        return __awaiter(this, void 0, void 0, function* () {
            switch (this.engine) {
                case databaseEngine.MariaDB:
                    const connector = new mariadb_connector_1.MariaDBConnector(this.ip, this.port, this.database, this.user, this.password);
                    yield connector.init();
                    return connector;
                default:
                    throw new Error('Unsupported engine.');
            }
        });
    }
    setHost(ip) {
        this.ip = ip;
        return this;
    }
    setPort(port) {
        this.port = port;
        return this;
    }
    setDatabase(database) {
        this.database = database;
        return this;
    }
    setCredentials(user, password) {
        this.user = user;
        this.password = password;
        return this;
    }
}
exports.DatabaseConnectorBuilder = DatabaseConnectorBuilder;
