"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const log4js_1 = require("log4js");
const clify_1 = require("@corteks/clify");
const fs = __importStar(require("fs-extra"));
const analyser_1 = require("./analysis/analyser");
const generator_1 = require("./generation/generator");
const database_connector_builder_1 = require("./database/database-connector-builder");
const customizer_1 = __importDefault(require("./analysis/customizer"));
const path = __importStar(require("path"));
const logger = log4js_1.getLogger();
logger.level = "debug";
let Main = class Main extends clify_1.CliMainClass {
    constructor() {
        super(...arguments);
        this.database = undefined;
        this.host = '127.0.0.1:3306';
        this.user = 'root';
        this.password = 'root';
        this.analyse = false;
        this.reset = false;
    }
    main() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.database)
                throw new Error('Please provide a valid database name');
            const [host, port] = this.host.split(':');
            const dbConnectorBuilder = new database_connector_builder_1.DatabaseConnectorBuilder(database_connector_builder_1.databaseEngine.MariaDB);
            let dbConnector;
            try {
                dbConnector = yield dbConnectorBuilder
                    .setHost(host)
                    .setPort(parseInt(port, 10))
                    .setDatabase(this.database)
                    .setCredentials(this.user, this.password)
                    .build();
            }
            catch (err) {
                logger.error(err.message);
                return 1;
            }
            if (!fs.pathExistsSync('settings')) {
                fs.mkdirSync('settings');
            }
            try {
                if (this.analyse) {
                    let customSchema = analyser_1.dummyCustomSchema;
                    try {
                        customSchema = fs.readJSONSync(path.join('settings', 'custom_schema.json'));
                    }
                    catch (ex) {
                        logger.warn('Unable to read ./settings/custom_schema.json, this will not take any customization into account.');
                    }
                    const customizer = new customizer_1.default(customSchema, logger);
                    const analyser = new analyser_1.Analyser(dbConnector, customSchema, customizer, logger);
                    const json = yield analyser.analyse();
                    fs.writeJSONSync(path.join('settings', 'schema.json'), json, { spaces: 4 });
                    return 0;
                }
                ;
                let schema = fs.readJSONSync(path.join('settings', 'schema.json'));
                const tableService = new generator_1.Generator(dbConnector, schema, logger);
                yield dbConnector.backupTriggers(schema.tables.filter(table => table.maxLines || table.addLines).map(table => table.name));
                /** @todo: Remove deprecated warning */
                let useDeprecatedLines = false;
                for (const table of schema.tables) {
                    if (table.lines) {
                        useDeprecatedLines = true;
                        table.maxLines = table.lines;
                    }
                    if (table.maxLines || table.addLines) {
                        yield tableService.fill(table, this.reset, schema.settings.disableTriggers);
                    }
                }
                if (useDeprecatedLines)
                    console.warn('DEPRECATED: Table.lines is deprecated, please use table.maxLines instead.');
                /****************/
                dbConnector.cleanBackupTriggers();
            }
            catch (ex) {
                if (ex.code == 'ENOENT') {
                    logger.error('Unable to read from ./settings/schema.json. Please run with --analyse first.');
                }
                else {
                    logger.error(ex);
                }
            }
            finally {
                logger.info('Close database connection');
                yield dbConnector.destroy();
            }
            return 0;
        });
    }
};
__decorate([
    clify_1.CliParameter({ alias: 'db', demandOption: true, description: 'database', }),
    __metadata("design:type", Object)
], Main.prototype, "database", void 0);
__decorate([
    clify_1.CliParameter({ alias: 'h' }),
    __metadata("design:type", String)
], Main.prototype, "host", void 0);
__decorate([
    clify_1.CliParameter(),
    __metadata("design:type", String)
], Main.prototype, "user", void 0);
__decorate([
    clify_1.CliParameter(),
    __metadata("design:type", String)
], Main.prototype, "password", void 0);
__decorate([
    clify_1.CliParameter(),
    __metadata("design:type", Boolean)
], Main.prototype, "analyse", void 0);
__decorate([
    clify_1.CliParameter(),
    __metadata("design:type", Boolean)
], Main.prototype, "reset", void 0);
Main = __decorate([
    clify_1.CliMain
], Main);
